@@ -51,6 +51,7 @@ sub build_select
   my $tables_sql  = $args{'tables_sql'} || $tables;
   my $logic       = delete $args{'logic'} || 'AND';
   my $columns     = $args{'columns'};  
+  my $all_columns = $args{'all_columns'} || {};
   my $query_arg   = delete $args{'query'};
   my $sort_by     = delete $args{'sort_by'};
   my $group_by    = delete $args{'group_by'};
@@ -64,6 +65,8 @@ sub build_select
   my $set         = delete $args{'set'};
   my $table_map   = delete $args{'table_map'} || {};
 
+  $all_columns = $columns  unless(%$all_columns);
+
   $logic = " $logic"  unless($logic eq ',');
 
   $args{'_depth'}++;
@@ -167,7 +170,7 @@ sub build_select
     my $table_tn    = $table_num;
     my $table_alias = 't' . $table_num++;
 
-    next  unless($columns->{$table});
+    #next  unless($all_columns->{$table} ||= $columns->{$table});
 
     my($classes, $meta, $obj_class, $obj_meta);
 
@@ -189,17 +192,18 @@ sub build_select
     }
 
     my $query_only_columns = 0;
-    my $columns = $columns->{$table};
+    my $my_columns     = $columns->{$table};
+    my $all_my_columns = $all_columns->{$table} ||= $my_columns;
 
     # No columns to select, but allow them to be queried if we can
-    if(@$columns == 0)
+    if(@$my_columns == 0)
     {
       # Don't select these columns, but allow them to participate in the query
       $query_only_columns = 1; 
 
       if($obj_meta)
       {
-        $columns = $obj_meta->column_names;
+        $my_columns = $all_my_columns = $obj_meta->column_names;
       }
       else # Try to dig out meta object even if query_is_sql
       {
@@ -210,12 +214,14 @@ sub build_select
 
         if($obj_meta)
         {
-          $columns = $obj_meta->column_names;
+          $my_columns = $obj_meta->column_names;
         }
       }
     }
 
-    foreach my $column (@$columns)
+    my %select_columns = map { $_ => 1 } @$my_columns;
+
+    foreach my $column (@$all_my_columns)
     {
       my $fq_column     = "$table.$column";
       my $short_column  = "$table_alias.$column";
@@ -225,7 +231,7 @@ sub build_select
 
       my $method = $obj_meta ? $obj_meta->column_rw_method_name($column) : undef;
 
-      unless($query_only_columns)
+      unless($query_only_columns || !$select_columns{$column})
       {
         push(@select_columns, $multi_table ? 
              "$short_column AS ${table_alias}_$column" : 
